@@ -2,7 +2,7 @@ using System;
 using System.Collections.Generic;
 using System.Windows.Forms;
 
-namespace Google_CalendarApplication
+namespace Google_SpreadsheetApplication
 {
     static class Program
     {
