@@ -29,34 +29,49 @@ namespace Google.GData.ContentForShopping {
 
         /// <summary>
         /// AdditionalImageLink collection
-        /// </summary> 
+        /// </summary>
         private ExtensionCollection<AdditionalImageLink> additionalImageLinks;
 
         /// <summary>
         /// Tax collection
-        /// </summary> 
+        /// </summary>
         private ExtensionCollection<Tax> taxRules;
 
         /// <summary>
         /// Shipping collection
-        /// </summary> 
+        /// </summary>
         private ExtensionCollection<Shipping> shippingRules;
 
-        /// <summary>
-        /// Color collection
-        /// </summary> 
-        private ExtensionCollection<Color> colors;
-
         /// <summary>
         /// Feature collection
-        /// </summary> 
+        /// </summary>
         private ExtensionCollection<Feature> features;
 
         /// <summary>
         /// Size collection
-        /// </summary> 
+        /// </summary>
         private ExtensionCollection<Size> sizes;
 
+        /// <summary>
+        /// AdwordsLabels collection
+        /// </summary>
+        private ExtensionCollection<AdwordsLabels> adwordsLabelsCollection;
+
+        /// <summary>
+        /// AdwordsQueryParam collection
+        /// </summary>
+        private ExtensionCollection<AdwordsQueryParam> adwordsQueryParams;
+
+        /// <summary>
+        /// Custom attributes collection
+        /// </summary>
+        private ExtensionCollection<CustomAttribute> customAttributes;
+
+        /// <summary>
+        /// Custom groups collection
+        /// </summary>
+        private ExtensionCollection<CustomGroup> customGroups;
+
         public ProductEntry()
             : base() {
             this.AddExtension(new ProductId());
@@ -65,6 +80,10 @@ namespace Google.GData.ContentForShopping {
             this.AddExtension(new ExpirationDate());
             this.AddExtension(new AdditionalImageLink());
             this.AddExtension(new Adult());
+            this.AddExtension(new AdwordsGrouping());
+            this.AddExtension(new AdwordsLabels());
+            this.AddExtension(new AdwordsQueryParam());
+            this.AddExtension(new AdwordsRedirect());
             this.AddExtension(new AgeGroup());
             this.AddExtension(new Author());
             this.AddExtension(new Availability());
@@ -72,6 +91,8 @@ namespace Google.GData.ContentForShopping {
             this.AddExtension(new Channel());
             this.AddExtension(new Color());
             this.AddExtension(new Condition());
+            this.AddExtension(new CustomAttribute());
+            this.AddExtension(new CustomGroup());
             this.AddExtension(new Edition());
             this.AddExtension(new Feature());
             this.AddExtension(new FeaturedProduct());
@@ -87,13 +108,16 @@ namespace Google.GData.ContentForShopping {
             this.AddExtension(new Mpn());
             this.AddExtension(new Pages());
             this.AddExtension(new Pattern());
+            this.AddExtension(new Performance());
             this.AddExtension(new Price());
             this.AddExtension(new ProductReviewAverage());
             this.AddExtension(new ProductReviewCount());
             this.AddExtension(new ProductType());
             this.AddExtension(new ProductWeight());
-            this.AddExtension(new Publisher()); 
+            this.AddExtension(new Publisher());
             this.AddExtension(new Quantity());
+            this.AddExtension(new SalePrice());
+            this.AddExtension(new SalePriceEffectiveDate());
             this.AddExtension(new Shipping());
             this.AddExtension(new ShippingWeight());
             this.AddExtension(new Size());
@@ -202,6 +226,36 @@ namespace Google.GData.ContentForShopping {
             }
         }
 
+        /// <summary>
+        /// Adwords Grouping
+        /// </summary>
+        public string AdwordsGrouping {
+            get {
+                return GetStringValue<AdwordsGrouping>(ContentForShoppingNameTable.AdwordsGrouping,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+            set {
+                SetStringValue<AdwordsGrouping>(value,
+                    ContentForShoppingNameTable.AdwordsGrouping,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+        }
+
+        /// <summary>
+        /// Adwords Redirect
+        /// </summary>
+        public string AdwordsRedirect {
+            get {
+                return GetStringValue<AdwordsRedirect>(ContentForShoppingNameTable.AdwordsRedirect,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+            set {
+                SetStringValue<AdwordsRedirect>(value,
+                    ContentForShoppingNameTable.AdwordsRedirect,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+        }
+
         /// <summary>
         /// AgeGroup.
         /// </summary>
@@ -293,6 +347,21 @@ namespace Google.GData.ContentForShopping {
             }
         }
 
+        /// <summary>
+        /// Color.
+        /// </summary>
+        public string Color {
+            get {
+                return GetStringValue<Color>(ContentForShoppingNameTable.Color,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+            set {
+                SetStringValue<Color>(value,
+                    ContentForShoppingNameTable.Color,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+        }
+
         /// <summary>
         /// Condition.
         /// </summary>
@@ -594,6 +663,51 @@ namespace Google.GData.ContentForShopping {
             }
         }
 
+        /// <summary>
+        /// Performance.
+        /// </summary>
+        public Performance Performance {
+            get {
+                return FindExtension(ContentForShoppingNameTable.Performance,
+                    ContentForShoppingNameTable.BaseNamespace) as Performance;
+            }
+            set {
+                ReplaceExtension(ContentForShoppingNameTable.Performance,
+                    ContentForShoppingNameTable.BaseNamespace,
+                    value);
+            }
+        }
+
+        /// <summary>
+        /// Sale Price.
+        /// </summary>
+        public SalePrice SalePrice {
+            get {
+                return FindExtension(ContentForShoppingNameTable.SalePrice,
+                    ContentForShoppingNameTable.ProductsNamespace) as SalePrice;
+            }
+            set {
+                ReplaceExtension(ContentForShoppingNameTable.SalePrice,
+                    ContentForShoppingNameTable.ProductsNamespace,
+                    value);
+            }
+        }
+
+        /// <summary>
+        /// Sale Price Effective Date.
+        /// </summary>
+        public string SalePriceEffectiveDate {
+            get {
+                return GetStringValue<SalePriceEffectiveDate>(ContentForShoppingNameTable.SalePriceEffectiveDate,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+            set {
+                SetStringValue<SalePriceEffectiveDate>(value,
+                    ContentForShoppingNameTable.SalePriceEffectiveDate,
+                    ContentForShoppingNameTable.ProductsNamespace);
+            }
+        }
+
         /// <summary>
         /// Shipping Weight.
         /// </summary>
@@ -675,18 +789,6 @@ namespace Google.GData.ContentForShopping {
             }
         }
 
-        /// <summary>
-        /// Color collection.
-        /// </summary>
-        public ExtensionCollection<Color> Colors {
-            get {
-                if (this.colors == null) {
-                    this.colors = new ExtensionCollection<Color>(this);
-                }
-                return this.colors;
-            }
-        }
-
         /// <summary>
         /// Feature collection.
         /// </summary>
@@ -710,5 +812,63 @@ namespace Google.GData.ContentForShopping {
                 return this.sizes;
             }
         }
+
+        /// <summary>
+        /// AdwordsLabels collection
+        /// </summary>
+        public ExtensionCollection<AdwordsLabels> AdwordsLabelsCollection {
+            get {
+                if (this.adwordsLabelsCollection == null) {
+                    this.adwordsLabelsCollection = new ExtensionCollection<AdwordsLabels>(this);
+                }
+                return this.adwordsLabelsCollection;
+            }
+        }
+
+        /// <summary>
+        /// AdwordsQueryParam collection
+        /// </summary>
+        public ExtensionCollection<AdwordsQueryParam> AdwordsQueryParams {
+            get {
+                if (this.adwordsQueryParams == null) {
+                    this.adwordsQueryParams = new ExtensionCollection<AdwordsQueryParam>(this);
+                }
+                return this.adwordsQueryParams;
+            }
+        }
+
+        /// <summary>
+        /// Custom attribute collection.
+        /// </summary>
+        public ExtensionCollection<CustomAttribute> CustomAttributes {
+            get {
+                if (this.customAttributes == null) {
+                    this.customAttributes = new ExtensionCollection<CustomAttribute>(this);
+                }
+                return this.customAttributes;
+            }
+        }
+
+        /// <summary>
+        /// Custom group collection.
+        /// </summary>
+        public ExtensionCollection<CustomGroup> Groups {
+            get {
+                if (this.customGroups == null) {
+                    this.customGroups = new ExtensionCollection<CustomGroup>(this);
+                }
+                return this.customGroups;
+            }
+        }
+
+        /// <summary>
+        /// gd:errors element
+        /// </summary>
+        /// <returns></returns>
+        public ExtensionCollection<BatchError> BatchErrors {
+            get {
+                return this.Content.BatchErrors.Errors;
+            }
+        }
     }
 }
