@@ -24,16 +24,12 @@ using Google.GData.Extensions;
 using System.Collections.Generic;
 using Google.GData.Extensions.AppControl;
 using System.Security.Cryptography;
-#if WindowsCE || PocketPC
-#else
 using System.ComponentModel;
-#endif
 
 namespace Google.GData.Client {
-    //////////////////////////////////////////////////////////////////////
-    /// <summary>a generic Feed class
+    /// <summary>
+    /// a generic Feed class
     /// </summary>
-    //////////////////////////////////////////////////////////////////////
     public class Feed<T> where T : Entry, new() {
         AtomFeed af;
         bool paging;
@@ -130,7 +126,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        /// the maxium number of entries to be retrieved. This is normally
+        /// the maximum number of entries to be retrieved. This is normally
         /// setup using the RequestSettings when the feed is constructed.
         /// </summary>
         /// <returns></returns>
@@ -156,38 +152,37 @@ namespace Google.GData.Client {
             }
         }
 
-        /**
-        <summary>
-         returns the initial list of entries.This page is the data
-         you got from the Requestobject and will remain constant.
-         Unless you set AutoPaging to true, in that case:
-         This will go back to the server and fetch data again if
-         needed. Example. If you pagesize is 30, you get an initial set of
-         30 entries. While enumerating, when reaching 30, the code will go
-         to the server and get the next 30 rows. It will continue to do so
-         until the server reports no more rows available.
-         Note that you should cache the entries returned in a list of your own
-         if you want to access them more than once, as this one does no caching on
-         it's own.
-         </summary>
-          <example>
-                 The following code illustrates a possible use of
-                 the <c>Entries</c> property:
-                 <code>
-                   YouTubeRequestSettings settings = new YouTubeRequestSettings("yourApp", "yourClient", "yourKey", "username", "pwd");
-                   YouTubeRequest f = new YouTubeRequest(settings);
-                   Feed&lt;Playlist&gt; feed = f.GetPlaylistsFeed(null);
-                   foreach (Vidoe v in feed.Entries)
-                 </code>
-             </example>
-         <returns></returns>
-         */
+        /// <summary>
+        /// returns the initial list of entries.This page is the data
+        /// you got from the Requestobject and will remain constant.
+        /// Unless you set AutoPaging to true, in that case:
+        /// This will go back to the server and fetch data again if
+        /// needed. Example. If you pagesize is 30, you get an initial set of
+        /// 30 entries. While enumerating, when reaching 30, the code will go
+        /// to the server and get the next 30 rows. It will continue to do so
+        /// until the server reports no more rows available.
+        /// Note that you should cache the entries returned in a list of your own
+        /// if you want to access them more than once, as this one does no caching on
+        /// it's own.
+        /// </summary>
+        /// <example>
+        /// The following code illustrates a possible use of
+        /// the <c>Entries</c> property:
+        /// <code>
+        /// YouTubeRequestSettings settings = new YouTubeRequestSettings("yourApp", "yourClient", "yourKey", "username", "pwd");
+        /// YouTubeRequest f = new YouTubeRequest(settings);
+        /// Feed&lt;Playlist&gt; feed = f.GetPlaylistsFeed(null);
+        /// foreach (Vidoe v in feed.Entries)
+        /// </code>
+        /// </example>
+        /// <returns></returns>
         public IEnumerable<T> Entries {
             get {
                 bool looping;
 
-                if (this.AtomFeed == null)
+                if (this.AtomFeed == null) {
                     yield break;
+                }
 
                 AtomFeed originalFeed = this.AtomFeed;
 
@@ -218,7 +213,6 @@ namespace Google.GData.Client {
             }
         }
     }
-    //end of public class Feed
 
     /// <summary>
     /// the Entry class is the base class for all Feed of T type feeds
@@ -235,7 +229,9 @@ namespace Google.GData.Client {
         public Entry() {
         }
 
-        /// <summary>override for ToString, returns the Entries Title</summary>
+        /// <summary>
+        /// override for ToString, returns the Entries Title
+        /// </summary>
         public override string ToString() {
             return this.Title;
         }
@@ -247,14 +243,11 @@ namespace Google.GData.Client {
         protected abstract void EnsureInnerObject();
 
         /// <summary>
-        ///  the original AtomEntry object that this object is standing in for
+        /// the original AtomEntry object that this object is standing in for
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The original AtomEntry object that this object is standing in for")]
-#endif
         public AtomEntry AtomEntry {
             get {
                 return this.e;
@@ -267,11 +260,8 @@ namespace Google.GData.Client {
         /// <summary>
         /// returns the Id of an entry
         /// </summary>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The unique Id of the entry")]
-#endif
         public string Id {
             get {
                 EnsureInnerObject();
@@ -287,11 +277,8 @@ namespace Google.GData.Client {
         /// returns the value of the self uri as a string
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The value of the self uri as a string")]
-#endif
         public string Self {
             get {
                 EnsureInnerObject();
@@ -306,11 +293,8 @@ namespace Google.GData.Client {
         /// the title of the Entry.
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("Specifies the title of the entry.")]
-#endif
         public virtual string Title {
             get {
                 EnsureInnerObject();
@@ -325,11 +309,8 @@ namespace Google.GData.Client {
         /// <summary>
         /// returns the appControl sublement
         /// </summary>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The AppControl subobject.")]
-#endif
         public AppControl AppControl {
             get {
                 EnsureInnerObject();
@@ -344,11 +325,8 @@ namespace Google.GData.Client {
         /// <summary>
         /// returns the appControl sublement
         /// </summary>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("Specifies if the entry is considered a draft entry.")]
-#endif
         public bool IsDraft {
             get {
                 EnsureInnerObject();
@@ -359,11 +337,8 @@ namespace Google.GData.Client {
         /// <summary>
         /// returns true, if the entry has an edit link
         /// </summary>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("If then entry has no edit uri, it is considered read only.")]
-#endif
         public bool ReadOnly {
             get {
                 EnsureInnerObject();
@@ -375,11 +350,8 @@ namespace Google.GData.Client {
         ///  returns the first author name in the atom.entry.authors collection
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("returns the first author name in the atom.entry.authors collection.")]
-#endif
         public string Author {
             get {
                 EnsureInnerObject();
@@ -405,11 +377,8 @@ namespace Google.GData.Client {
         /// returns the string representation of the atom.content element
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("returns the string representation of the atom.content element.")]
-#endif
         public string Content {
             get {
                 EnsureInnerObject();
@@ -425,11 +394,8 @@ namespace Google.GData.Client {
         /// returns the string representation of the atom.Summary element
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("returns the string representation of the atom.Summary element.")]
-#endif
         public string Summary {
             get {
                 EnsureInnerObject();
@@ -445,11 +411,8 @@ namespace Google.GData.Client {
         /// just a thin layer on top of the existing updated of the
         /// underlying atomentry
         /// </summary>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The datetime at which the entry was updated the last time.")]
-#endif
         public DateTime Updated {
             get {
                 EnsureInnerObject();
@@ -465,11 +428,8 @@ namespace Google.GData.Client {
         /// this returns the batch data for the inner atom object
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The batchdata subobject.")]
-#endif
         public GDataBatchEntryData BatchData {
             get {
                 EnsureInnerObject();
@@ -485,11 +445,8 @@ namespace Google.GData.Client {
         /// returns the categories for the entry
         /// </summary>
         /// <returns></returns>
-#if WindowsCE || PocketPC
-#else
         [Category("Basic Entry Data"),
         Description("The Categories collection.")]
-#endif
         public AtomCategoryCollection Categories {
             get {
                 EnsureInnerObject();
@@ -497,24 +454,20 @@ namespace Google.GData.Client {
             }
         }
 
-        //////////////////////////////////////////////////////////////////////
         /// <summary>access the associated media element. Note, that setting this
         /// WILL cause subsequent updates to be done using MIME multipart posts
         /// </summary>
         /// <returns> </returns>
-        //////////////////////////////////////////////////////////////////////
-#if WindowsCE || PocketPC
-#else
         [Category("Media Data"),
         Description("The Mediasource subobject.")]
-#endif
         public MediaSource MediaSource {
             get {
                 EnsureInnerObject();
                 AbstractEntry ae = this.e as AbstractEntry;
 
-                if (ae != null)
+                if (ae != null) {
                     return ae.MediaSource;
+                }
 
                 return null;
             }
@@ -522,32 +475,29 @@ namespace Google.GData.Client {
                 EnsureInnerObject();
                 AbstractEntry ae = this.e as AbstractEntry;
 
-                if (ae != null)
+                if (ae != null) {
                     ae.MediaSource = value;
-                else
+                } else {
                     throw new InvalidOperationException("The AtomEntry contained does not support Media operations");
+                }
             }
         }
-        // end of accessor public MediaSource Media
 
-        //////////////////////////////////////////////////////////////////////
-        /// <summary>access the associated media element. Note, that setting this
+        /// <summary>
+        /// access the associated media element. Note, that setting this
         /// WILL cause subsequent updates to be done using MIME multipart posts
         /// </summary>
         /// <returns> </returns>
-        //////////////////////////////////////////////////////////////////////
-#if WindowsCE || PocketPC
-#else
         [Category("State Data"),
         Description("The etag information.")]
-#endif
         public string ETag {
             get {
                 EnsureInnerObject();
                 AbstractEntry ae = this.e as AbstractEntry;
 
-                if (ae != null)
+                if (ae != null) {
                     return ae.Etag;
+                }
 
                 return null;
             }
@@ -555,13 +505,13 @@ namespace Google.GData.Client {
                 EnsureInnerObject();
                 AbstractEntry ae = this.e as AbstractEntry;
 
-                if (ae != null)
+                if (ae != null) {
                     ae.Etag = value;
-                else
+                } else {
                     throw new InvalidOperationException("The AtomEntry contained does not support ETags operations");
+                }
             }
         }
-        // end of accessor public MediaSource Media
     }
 
     /// <summary>
@@ -575,7 +525,8 @@ namespace Google.GData.Client {
             none,
             clientLogin,
             authSub,
-            oAuth
+            oAuth,
+            oAuth2
         }
 
         private AuthenticationType authType = AuthenticationType.none;
@@ -606,18 +557,18 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        ///  a constructor for client login use cases
+        /// a constructor for client login use cases
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
         /// <param name="userName">the user name</param>
         /// <param name="passWord">the password</param>
         /// <returns></returns>
-        public RequestSettings(string applicationName, string userName, string passWord) :
-            this(applicationName, new GDataCredentials(userName, passWord)) {
+        public RequestSettings(string applicationName, string username, string password) :
+            this(applicationName, new GDataCredentials(username, password)) {
         }
 
         /// <summary>
-        ///  a constructor for OpenAuthentication login use cases
+        /// a constructor for OpenAuthentication login use cases
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
         /// <param name="consumerKey">the consumerKey to use</param>
@@ -640,7 +591,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        ///  a constructor for OpenAuthentication login use cases using 2 or 3 legged oAuth
+        /// a constructor for OpenAuthentication login use cases using 2 or 3-legged OAuth
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
         /// <param name="consumerKey">the consumerKey to use</param>
@@ -664,7 +615,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        ///  a constructor for client login use cases
+        /// a constructor for client login use cases
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
         /// <param name="credentials">the user credentials</param>
@@ -704,6 +655,18 @@ namespace Google.GData.Client {
             this.authSubToken = authSubToken;
         }
 
+        /// <summary>
+        /// a constructor for a OAuth 2.0 authorization scenario.
+        /// <param name="applicationName"></param>
+        /// <param name="parameters">OAuth 2.0 parameters</param>
+        public RequestSettings(
+            string applicationName,
+            OAuth2Parameters parameters)
+            : this(applicationName) {
+            this.authType = AuthenticationType.oAuth2;
+            this.OAuth2Parameters = parameters;
+        }
+
         /// <summary>
         /// returns the Credentials in case of a client login scenario
         /// </summary>
@@ -765,7 +728,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        /// returns the Token for oAuth
+        /// returns the Token for OAuth
         /// </summary>
         /// <returns></returns>
         public string Token {
@@ -775,7 +738,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        /// returns the TokenSecret for oAuth
+        /// returns the TokenSecret for OAuth
         /// </summary>
         /// <returns></returns>
         public string TokenSecret {
@@ -833,7 +796,6 @@ namespace Google.GData.Client {
         /// AutoPaging specifies if a feed iterator should return to the server to fetch more data
         /// automatically. If set to false, a loop over feed.Entries will stop when the currently
         /// fetched set of data reaches it's end.  This is false by default. <seealso cref="RequestSettings.Maximum"/>
-        ///
         /// </summary>
         ///  <example>
         ///         The following code illustrates a possible use of
@@ -856,8 +818,8 @@ namespace Google.GData.Client {
         /// <summary>
         /// the Maximum specifies how many entries should be retrieved in total. This works together with
         /// <seealso cref="RequestSettings.AutoPaging"/>. If set, AutoPaging of a feed will stop when the
-        /// specified amount of entries was iterated over. If Maximum is smaller than  PageSize (<seealso cref="RequestSettings.PageSize"/>),
-        ///  an exception is thrown. The default is -1 (ignored).
+        /// specified amount of entries was iterated over. If Maximum is smaller than PageSize (<seealso cref="RequestSettings.PageSize"/>),
+        /// an exception is thrown. The default is -1 (ignored).
         /// </summary>
         ///  <example>
         ///         The following code illustrates a possible use of
@@ -926,12 +888,11 @@ namespace Google.GData.Client {
             }
         }
 
-        //////////////////////////////////////////////////////////////////////
-        /// <summary>ClientLoginHandler - this is the URI that is used to
+        /// <summary>
+        /// ClientLoginHandler - this is the URI that is used to
         /// retrieve a client login authentication token
         /// </summary>
         /// <returns> </returns>
-        //////////////////////////////////////////////////////////////////////
         public Uri ClientLoginHandler {
             get {
                 return this.clientLoginHandler != null ?
@@ -940,6 +901,8 @@ namespace Google.GData.Client {
             set { this.clientLoginHandler = value; }
         }
 
+        public OAuth2Parameters OAuth2Parameters { get; set; }
+
         /// <summary>
         /// Creates a HttpWebRequest object that can be used against a given service.
         /// for a RequestSetting object that is using client login, this might call
@@ -961,24 +924,25 @@ namespace Google.GData.Client {
             if (request == null) {
                 throw new ArgumentException("targetUri does not resolve to an http request");
             }
+
             if (this.authType == AuthenticationType.clientLogin) {
                 EnsureClientLoginCredentials(request, serviceName);
             }
-#if WindowsCE || PocketPC
-#else
+
             if (this.authType == AuthenticationType.authSub) {
                 EnsureAuthSubCredentials(request);
             }
+
             if (this.authType == AuthenticationType.oAuth) {
                 EnsureOAuthCredentials(request);
             }
-#endif
+
             return request;
         }
 
         private void EnsureClientLoginCredentials(HttpWebRequest request, string serviceName) {
             if (String.IsNullOrEmpty(this.Credentials.ClientToken)) {
-                this.Credentials.ClientToken = 
+                this.Credentials.ClientToken =
                     Utilities.QueryClientLoginToken(
                     this.Credentials,
                     serviceName,
@@ -986,13 +950,13 @@ namespace Google.GData.Client {
                     false,
                     this.ClientLoginHandler);
             }
+
             if (!String.IsNullOrEmpty(this.Credentials.ClientToken)) {
                 string strHeader = GoogleAuthentication.Header + this.Credentials.ClientToken;
                 request.Headers.Add(strHeader);
             }
         }
-#if WindowsCE || PocketPC
-#else
+
         private void EnsureAuthSubCredentials(HttpWebRequest request) {
             string header = AuthSubUtil.formAuthorizationHeader(
                 this.Token,
@@ -1012,7 +976,6 @@ namespace Google.GData.Client {
                 request.Method);
             request.Headers.Add(oauthHeader);
         }
-#endif
     }
 
     /// <summary>
@@ -1065,8 +1028,7 @@ namespace Google.GData.Client {
             if (settings.Credentials != null) {
                 s.Credentials = settings.Credentials;
             }
-#if WindowsCE || PocketPC
-#else
+
             if (settings.AuthSubToken != null) {
                 GAuthSubRequestFactory authFactory = new GAuthSubRequestFactory(s.ServiceIdentifier, settings.Application);
                 authFactory.UserAgent = authFactory.UserAgent + "--IEnumerable";
@@ -1074,13 +1036,15 @@ namespace Google.GData.Client {
                 authFactory.PrivateKey = settings.PrivateKey;
                 s.RequestFactory = authFactory;
             } else if (settings.ConsumerKey != null) {
-                // let's create an oauth factory
+                // let's create an OAuth factory
                 GOAuthRequestFactory authFactory = new GOAuthRequestFactory(s.ServiceIdentifier, settings.Application);
                 authFactory.ConsumerKey = settings.ConsumerKey;
                 authFactory.ConsumerSecret = settings.ConsumerSecret;
                 authFactory.Token = settings.Token;
                 authFactory.TokenSecret = settings.TokenSecret;
                 s.RequestFactory = authFactory;
+            } else if (settings.OAuth2Parameters != null) {
+                s.RequestFactory = new GOAuth2RequestFactory(s.ServiceIdentifier, settings.Application, settings.OAuth2Parameters);
             } else {
                 GDataGAuthRequestFactory authFactory = s.RequestFactory as GDataGAuthRequestFactory;
                 if (authFactory != null) {
@@ -1096,7 +1060,6 @@ namespace Google.GData.Client {
             }
 
             s.RequestFactory.UseSSL = settings.UseSSL;
-#endif
         }
 
         /// <summary>
@@ -1128,9 +1091,6 @@ namespace Google.GData.Client {
         /// <param name="location"></param>
         /// <returns></returns>
         protected Uri CreateUri(string location) {
-#if WindowsCE || PocketPC
-                return new Uri(location);
-#else
             Uri retUri = null;
 
             if (this.settings.OAuthUser != null && location.IndexOf(OAuthUri.OAuthParameter) != 0) {
@@ -1140,7 +1100,6 @@ namespace Google.GData.Client {
             }
 
             return retUri;
-#endif
         }
 
         /// <summary>
@@ -1220,9 +1179,7 @@ namespace Google.GData.Client {
         protected virtual void OnSetOtherProxies(IWebProxy proxy) {
         }
 
-        /// <summary>
-        ///
-        /// </summary>
+        /// <summary></summary>
         /// <returns></returns>
         protected Service AtomService {
             get {
@@ -1270,6 +1227,7 @@ namespace Google.GData.Client {
                     spec = feed.AtomFeed.Self;
                     break;
             }
+
             if (!String.IsNullOrEmpty(spec)) {
                 FeedQuery q = new FeedQuery(spec);
                 if (operation == FeedRequestType.Refresh) {
@@ -1367,6 +1325,7 @@ namespace Google.GData.Client {
                 Feed<Y> f = new Feed<Y>(resultFeed);
                 return f;
             }
+
             return null;
         }
 
@@ -1452,7 +1411,7 @@ namespace Google.GData.Client {
         }
 
         /// <summary>
-        ///  sends the data back to the server.
+        /// sends the data back to the server.
         /// </summary>
         /// <returns>the reflected entry from the server if any given</returns>
         public Y Update<Y>(Y entry) where Y : Entry, new() {
@@ -1474,11 +1433,12 @@ namespace Google.GData.Client {
                 r = new Y();
                 r.AtomEntry = ae;
             }
+
             return r;
         }
 
         /// <summary>
-        ///  deletes the Entry from the Server
+        /// deletes the Entry from the Server
         /// </summary>
         public void Delete<Y>(Y entry) where Y : Entry, new() {
             if (entry == null) {
@@ -1489,6 +1449,10 @@ namespace Google.GData.Client {
                 throw new ArgumentNullException("Entry.AtomEntry was null");
             }
 
+            if (entry.AtomEntry.EditUri == null) {
+                throw new ArgumentNullException("The AtomEntry has no EditUri");
+            }
+
             FeedQuery q = PrepareQuery<FeedQuery>(entry.AtomEntry.EditUri.ToString());
             this.Service.Delete(q.Uri, entry.ETag);
         }
@@ -1516,11 +1480,14 @@ namespace Google.GData.Client {
             }
 
             Y r = null;
-            AtomEntry ae = this.Service.Insert(address, entry.AtomEntry);
+            FeedQuery q = PrepareQuery<FeedQuery>(address.AbsoluteUri);
+
+            AtomEntry ae = this.Service.Insert(q.Uri, entry.AtomEntry);
             if (ae != null) {
                 r = new Y();
                 r.AtomEntry = ae;
             }
+
             return r;
         }
 
@@ -1529,27 +1496,7 @@ namespace Google.GData.Client {
         /// </summary>
         /// <returns>the reflected entry from the server if any given</returns>
         public Y Insert<Y>(Feed<Y> feed, Y entry) where Y : Entry, new() {
-            if (entry == null) {
-                throw new ArgumentNullException("Entry was null");
-            }
-
-            if (entry.AtomEntry == null) {
-                throw new ArgumentNullException("Entry.AtomEntry was null");
-            }
-
-            if (feed == null) {
-                throw new ArgumentNullException("Feed was null");
-            }
-
-            Y r = null;
-            FeedQuery q = PrepareQuery<FeedQuery>(feed.AtomFeed.Post);
-
-            AtomEntry ae = this.Service.Insert(q.Uri, entry.AtomEntry);
-            if (ae != null) {
-                r = new Y();
-                r.AtomEntry = ae;
-            }
-            return r;
+            return Insert(new Uri(feed.AtomFeed.Post), entry);
         }
 
         /// <summary>
