@@ -37,11 +37,8 @@ using System.Runtime.InteropServices;
 
 [assembly: ComVisible(false)]
 [assembly: CLSCompliant(true)]
-#if WindowsCE || PocketPC
-#else
 [assembly:AllowPartiallyTrustedCallers]
 [assembly: AssemblyDescription("Release Build: This assembly is marked as partially trustable. Use on your webserver at your own risk")]
-#endif
 
 //
 // Version information for an assembly consists of the following four values:
@@ -54,7 +51,7 @@ using System.Runtime.InteropServices;
 // You can specify all the values or you can default the Revision and Build Numbers 
 // by using the '*' as shown below:
 
-[assembly: AssemblyVersion("1.9.0.*")]
+[assembly: AssemblyVersion("2.1.0.*")]
 
 
 
